/*:
 * @target MZ
 * @plugindesc Simple Day/Night Cycle - Street Lighting
 * @author Your Name
 * @help Adds a simple day/night cycle with lighting on a specific map.
 *
 * Map Note: <StreetDayNight> - Tag to enable day/night cycle on this map.
 *
 * @param Minutes Per Step
 * @desc How many minutes pass for each map update.
 * @default 30
 *
 * @param Day Start Hour
 * @desc The hour when day starts (0-23).
 * @default 6
 *
 * @param Night Start Hour
 * @desc The hour when night starts (0-23).
 * @default 18
 *
 * @param Tint Duration
 * @desc The duration of the tint effect in frames.
 * @default 120
 */

(function() {

  const parameters = PluginManager.parameters('StreetDayNightCycle');
  const minutesPerStep = parseInt(parameters['Minutes Per Step'] || 30);
  const dayStartHour = parseInt(parameters['Day Start Hour'] || 6);
  const nightStartHour = parseInt(parameters['Night Start Hour'] || 18);
  const tintDuration = parseInt(parameters['Tint Duration'] || 120);

  const dayTone = [0, 0, 0, 0];
  const nightTone = [-155, -155, -155, 0];

  // ** Game_Map **
  const _Game_Map_setup = Game_Map.prototype.setup;
  Game_Map.prototype.setup = function(mapId) {
    _Game_Map_setup.call(this, mapId);

    // Инициализируем totalMinutes только на карте с тегом <StreetDayNight>
    if ($dataMap && $dataMap.note && $dataMap.note.includes("<StreetDayNight>")) {
      this._totalMinutes = 0;  // Добавляем totalMinutes к Game_Map
    } else {
      this._totalMinutes = null; // Устанавливаем в null, чтобы не работало на других картах
    }
  };

  const _Game_Map_update = Game_Map.prototype.update;
  Game_Map.prototype.update = function(sceneActive) {
    _Game_Map_update.call(this, sceneActive);

    if ($dataMap && $dataMap.note && $dataMap.note.includes("<StreetDayNight>") && this._totalMinutes !== null) {
      this._totalMinutes += minutesPerStep;  // Используем this._totalMinutes

      let tone = dayTone;
      if (isNight(this._totalMinutes)) {  // Передаём totalMinutes в функцию isNight
        tone = nightTone;
      }

      $gameScreen.startTint(tone, tintDuration);
    }
  };

  function isNight(totalMinutes) { // Принимаем totalMinutes в качестве аргумента
    const currentHour = Math.floor((totalMinutes / 60) % 24);
    return currentHour >= nightStartHour || currentHour < dayStartHour;
  }

})();